﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Store.BusinessLogic.Model;

namespace Store.BusinessLogic.Managers
{
    public class CustomerManager : ManagerBase, ICustomerManager
    {
        public CustomerManager() : this(null) { }

        ///
        public CustomerManager(StoreEntityModelContainer db) : base(db)
        {
            // do nothing
        }

        public IEnumerable<Customer> List()
        {
            return DatabaseContext.Customers.ToList();
        }

        public Customer Get(int id)
        {
            return DatabaseContext.Customers.FirstOrDefault(c => c.Id == id);
        }

        public Customer Get(string userName)
        {
            if (userName == null)
                throw new ArgumentNullException("userName");
            if (string.IsNullOrEmpty(userName))
                throw new ArgumentException("User name can't be empty.");

            return DatabaseContext.Customers.FirstOrDefault(c => c.UserName.ToLower() == userName.ToLower());
        }

        public void Create(Customer customer)
        {
            if (customer == null)
                throw new ArgumentNullException("customer");

            DatabaseContext.Customers.AddObject(customer);
        }

        public void Update(Customer customer)
        {
            if (customer == null)
                throw new ArgumentNullException("customer");

            Customer originalCustomer = Get(customer.Id);
            DatabaseContext.Customers.ApplyCurrentValues(customer);
        }

        public void Delete(Customer customer)
        {
            if (customer == null)
                throw new ArgumentNullException("customer");

            Customer customerToDelete = new Customer();
            DatabaseContext.Customers.DeleteObject(customerToDelete);
        }
    }
}
