﻿using System;
using System.Linq;
using Store.BusinessLogic.Model;
using System.Diagnostics;

namespace Store.BusinessLogic.Managers
{
    public class CustomerMembershipFacade : ManagerBase
    {
        private readonly IMembershipManager membershipManager;

        /// <exception cref="ArgumentNullException"><paramref name="membershipManager" /> is <c>null</c>.</exception>
        public CustomerMembershipFacade(IMembershipManager membershipManager)
        {
            if (membershipManager == null)
                throw new ArgumentNullException("membershipManager");

            this.membershipManager = membershipManager;
        }

        public bool Register(string userName, string password, string email, string fullName)
        {
            if (DatabaseContext.Customers.Any(x => x.UserName.ToLower() == userName.ToLower()))
                return false;

            if (!membershipManager.CreateUser(userName, password, email))
                return false;

            Customer customer = new Customer
            {
                UserName = userName,
                Name = fullName
            };
            DatabaseContext.Customers.AddObject(customer);
            Save();

            return true;
        }
    }
}