﻿using System;
using Store.BusinessLogic.Model;
using System.Data.Objects;

namespace Store.BusinessLogic.Managers
{
    public class ManagerBase : IDisposable
    {
        private readonly StoreEntityModelContainer db;
        private readonly bool isPrivateContainer = false;
        private bool isDisposed;

        public StoreEntityModelContainer DatabaseContext
        {
            get { return db; }
        }

        public ManagerBase()
            : this(null)
        {
        }

        public ManagerBase(StoreEntityModelContainer dbContext)
        {
            db = dbContext ?? new StoreEntityModelContainer();
            isPrivateContainer = dbContext == null;
        }

        public void Save()
        {
            db.SaveChanges();
        }

        ~ManagerBase()
        {
            Dispose(false);
            GC.SuppressFinalize(this);
        }

        public void Dispose()
        {
            Dispose(true);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (isDisposed)
                return;

            if (disposing)
            {
                if (isPrivateContainer && db != null)
                {
                    db.Dispose();
                }
            }
            isDisposed = true;
        }
    }

    public class SelfSufficientManagerBase : IDisposable
    {
        private readonly StoreEntityModelContainer db;
        private readonly bool isPrivateContainer;
        private bool isDisposed;
        private bool saveChanges;

        public StoreEntityModelContainer DatabaseContext
        {
            get { return db; }
        }

        public SelfSufficientManagerBase()
            : this(null)
        {
        }

        public SelfSufficientManagerBase(StoreEntityModelContainer dbContext)
        {
            db = dbContext ?? new StoreEntityModelContainer();
            isPrivateContainer = dbContext == null;
        }

        #region IDisposable Members

        public void Dispose()
        {
            Dispose(true);
        }

        #endregion

        ~SelfSufficientManagerBase()
        {
            Dispose(false);
            GC.SuppressFinalize(this);
        }

        public void SaveChanges()
        {
            saveChanges = true;
        }

        private void Dispose(bool disposing)
        {
            if (isDisposed)
                return;

            if (disposing)
            {
                if (isPrivateContainer && db != null)
                {
                    if (saveChanges)
                    {
                        db.SaveChanges();
                    }
                    db.Dispose();
                }
            }
            isDisposed = true;
        }
    }
}