﻿/*
 * UoW and Moe transactional processing framework (principle demo)
 * 
 * JP|Softworks
 * Jiri Polasek 2011
 */

using System;
using Store.BusinessLogic.Model;
using System.Transactions;
using System.Data.Objects;

namespace Store.BusinessLogic.Managers
{
    public class StoreUnitOfWork : IStoreUnitOfWork
    {
        private readonly StoreEntityModelContainer context;
        private readonly bool isContextPrivate;
        private bool disposed;

        private IProductManager productManager;
        private IOrderManager orderManager;
        private ICustomerManager customerManager;

        public IProductManager ProductManager
        {
            get { return productManager ?? (productManager = new ProductManager(context)); }
        }

        public IOrderManager OrderManager
        {
            get { return orderManager ?? (orderManager = new OrderManager(context)); }
        }

        public ICustomerManager CustomerManager
        {
            get { return customerManager ?? (customerManager = new CustomerManager(context)); }
        }

        public StoreUnitOfWork()
            : this(null)
        {
        }

        public StoreUnitOfWork(StoreEntityModelContainer existingContext)
        {
            context = existingContext ?? new StoreEntityModelContainer();
            isContextPrivate = existingContext == null;
        }

        ~StoreUnitOfWork()
        {
            Dispose(false);
        }

        public void Save()
        {
            context.SaveChanges();
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if (!disposed)
            {
                if (disposing)
                {
                    if (isContextPrivate && context != null)
                    {
                        context.Dispose();
                    }
                }
            }

            disposed = true;
        }
    }
}