﻿using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using Store.Web.Models;
using Store.Web.Code;

namespace Store.Web
{
    // Note: For instructions on enabling IIS6 or IIS7 classic mode, 
    // visit http://go.microsoft.com/?LinkId=9394801

    public class MvcApplication : HttpApplication
    {
        public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new ElmahHandleErrorAttribute());
            filters.Add(new HandleErrorAttribute());
        }

        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            routes.MapRoute(
                "PageableProducts",
                "Product/Index/{page}",
                new {controller = "Product", action = "Index", page = 1}
                );

            routes.MapRoute(
                "Default", // Route name
                "{controller}/{action}/{id}", // URL with parameters
                new {controller = "Product", action = "Index", id = UrlParameter.Optional} // Parameter defaults
                );
        }

        protected void Application_Start()
        {
            AreaRegistration.RegisterAllAreas();

            RegisterUsers();

            RegisterGlobalFilters(GlobalFilters.Filters);
            RegisterRoutes(RouteTable.Routes);
        }

        private static void RegisterUsers()
        {
            if (!Roles.RoleExists(AccountMembershipService.AdministratorRoleName))
            {
                Roles.CreateRole(AccountMembershipService.AdministratorRoleName);
            }

            if (!Roles.RoleExists(AccountMembershipService.CustomersRoleName))
            {
                Roles.CreateRole(AccountMembershipService.CustomersRoleName);
            }

            if (Membership.GetUser("Admin", false) == null)
            {
                MembershipCreateStatus status;
                Membership.Provider.CreateUser("Admin", "admin", "admin@admin.com", null, null, true, null, out status);
            }

            if (Membership.GetUser("Admin", false) != null && !Roles.IsUserInRole("Admin", AccountMembershipService.AdministratorRoleName))
            {
                Roles.AddUserToRole("Admin", AccountMembershipService.AdministratorRoleName);
            }
        }
    }
}